import React, { useRef, useEffect, useState } from "react";
import startPoint from "../../../../assets/gltf-glb/arrow_green.glb";
import startEnd from "../../../../assets/gltf-glb/arrow_red.glb";
import { useGLTF } from "@react-three/drei";
import { useSelectedEventSphere } from "../../../../store/simulation/useSimulationStore";
import * as THREE from "three";
import { useThree } from "@react-three/fiber";

type VehicleUIProps = {
  vehicleStatusSample: VehicleEventSchema[];
  setVehicleStatusSample: React.Dispatch<
    React.SetStateAction<VehicleEventSchema[]>
  >;
};

const VehicleUI: React.FC<VehicleUIProps> = ({
  vehicleStatusSample,
  setVehicleStatusSample,
}) => {
  const { scene: startScene } = useGLTF(startPoint) as any;
  const { scene: endScene } = useGLTF(startEnd) as any;
  const { camera, gl, controls } = useThree();
  const { selectedEventSphere } = useSelectedEventSphere();

  const startMarker = useRef<THREE.Group>(null);
  const endMarker = useRef<THREE.Group>(null);
  const hasInitialized = useRef(false);

  const [draggedMarker, setDraggedMarker] = useState<"start" | "end" | null>(
    null
  );
  const [dragOffset, setDragOffset] = useState<THREE.Vector3 | null>(null);
  const [isRotating, setIsRotating] = useState(false);

  const raycaster = useRef(new THREE.Raycaster());
  const plane = useRef(new THREE.Plane(new THREE.Vector3(0, 1, 0), 0)); // Y = 0 plane
  const mouse = useRef(new THREE.Vector2());
  const prevMousePos = useRef({ x: 0, y: 0 });

  // Initialize start/end markers
  useEffect(() => {
    if (
      selectedEventSphere &&
      startMarker.current &&
      endMarker.current &&
      !hasInitialized.current
    ) {
      startMarker.current.clear();
      endMarker.current.clear();

      const startClone = startScene.clone();
      const endClone = endScene.clone();

      startClone.name = "start-marker";
      endClone.name = "end-marker";

      startClone.traverse((child: any) => {
        if (child.isMesh && child.name.toLowerCase().includes("handle")) {
          child.name = "handle";
        }
      });
      endClone.traverse((child: any) => {
        if (child.isMesh && child.name.toLowerCase().includes("handle")) {
          child.name = "handle";
        }
      });

      startMarker.current.add(startClone);
      endMarker.current.add(endClone);

      hasInitialized.current = true;
    }
  }, [selectedEventSphere, startScene, endScene]);

  // Position start/end markers
  useEffect(() => {
    if (!selectedEventSphere || !startMarker.current || !endMarker.current)
      return;

    const selectedVehicle = vehicleStatusSample.find(
      (vehicle) => vehicle.modelUuid === selectedEventSphere.userData.modelUuid
    );

    if (selectedVehicle?.point?.action) {
      const { pickUpPoint, unLoadPoint } = selectedVehicle.point.action;

      // Update start marker position
      if (pickUpPoint) {
        const localPos = new THREE.Vector3(
          pickUpPoint.x,
          pickUpPoint.y,
          pickUpPoint.z
        );
        const worldPos = selectedEventSphere.localToWorld(localPos);
        worldPos.y = 0; // Force y to 0
        startMarker.current.position.copy(worldPos);
      } else {
        const defaultLocal = new THREE.Vector3(0, 0, 1.5);
        const defaultWorld = selectedEventSphere.localToWorld(defaultLocal);
        defaultWorld.y = 0; // Force y to 0
        startMarker.current.position.copy(defaultWorld);
      }

      // Update end marker position
      if (unLoadPoint) {
        const localPos = new THREE.Vector3(
          unLoadPoint.x,
          unLoadPoint.y,
          unLoadPoint.z
        );
        const worldPos = selectedEventSphere.localToWorld(localPos);
        worldPos.y = 0; // Force y to 0
        endMarker.current.position.copy(worldPos);
      } else {
        const defaultLocal = new THREE.Vector3(0, 0, -1.5);
        const defaultWorld = selectedEventSphere.localToWorld(defaultLocal);
        defaultWorld.y = 0; // Force y to 0
        endMarker.current.position.copy(defaultWorld);
      }
    }
  }, [selectedEventSphere, vehicleStatusSample]);

  // Handle dragging and rotation
  const handlePointerDown = (e: any, markerType: "start" | "end") => {
    if (!selectedEventSphere) return;

    if (e.object.name === "handle") {
      setIsRotating(true);
      prevMousePos.current = { x: e.clientX, y: e.clientY };
      if (controls) (controls as any).enabled = false;
      e.stopPropagation();
      setDraggedMarker(markerType);
      return;
    }

    setDraggedMarker(markerType);
    if (controls) (controls as any).enabled = false;

    const marker =
      markerType === "start" ? startMarker.current : endMarker.current;
    if (!marker) return;

    mouse.current.x = (e.clientX / gl.domElement.clientWidth) * 2 - 1;
    mouse.current.y = -(e.clientY / gl.domElement.clientHeight) * 2 + 1;

    raycaster.current.setFromCamera(mouse.current, camera);

    const intersectPoint = new THREE.Vector3();
    raycaster.current.ray.intersectPlane(plane.current, intersectPoint);

    const offset = new THREE.Vector3().subVectors(
      marker.position,
      intersectPoint
    );
    setDragOffset(offset);
  };

  const handlePointerMove = (e: PointerEvent) => {
    if (!selectedEventSphere) return;

    if (isRotating) {
      const deltaX = e.clientX - prevMousePos.current.x;
      prevMousePos.current = { x: e.clientX, y: e.clientY };

      const rotationSpeed = 0.01;
      const marker =
        draggedMarker === "start" ? startMarker.current : endMarker.current;

      if (marker) {
        marker.rotation.y -= deltaX * rotationSpeed;
      }
      return;
    }

    if (!draggedMarker || !dragOffset) return;

    mouse.current.x = (e.clientX / gl.domElement.clientWidth) * 2 - 1;
    mouse.current.y = -(e.clientY / gl.domElement.clientHeight) * 2 + 1;

    raycaster.current.setFromCamera(mouse.current, camera);

    const intersectPoint = new THREE.Vector3();
    raycaster.current.ray.intersectPlane(plane.current, intersectPoint);

    if (!intersectPoint) return;

    const newPos = {
      x: intersectPoint.x + dragOffset.x,
      y: 0,
      z: intersectPoint.z + dragOffset.z,
    };

    if (draggedMarker === "start" && startMarker.current) {
      startMarker.current.position.set(newPos.x, newPos.y, newPos.z);
    } else if (draggedMarker === "end" && endMarker.current) {
      endMarker.current.position.set(newPos.x, newPos.y, newPos.z);
    }
  };

  const handlePointerUp = () => {
    if (isRotating) {
      setIsRotating(false);
      if (controls) (controls as any).enabled = true;
      return;
    }

    if (!selectedEventSphere || !draggedMarker || !dragOffset) {
      if (controls) (controls as any).enabled = true;
      return;
    }

    if (controls) (controls as any).enabled = true;

    const marker =
      draggedMarker === "start" ? startMarker.current : endMarker.current;
    if (!marker) return;

    const worldPos = marker.position.clone();
    const localPos = selectedEventSphere.worldToLocal(worldPos);

    // Direct update (no snapping, ground level forced at y = 0)
    const updatedLocalPos = { x: localPos.x, y: 0, z: localPos.z };

    setVehicleStatusSample((prev) =>
      prev.map((vehicle) => {
        if (
          vehicle.modelUuid === selectedEventSphere.userData.modelUuid &&
          selectedEventSphere
        ) {
          const updatedVehicle = {
            ...vehicle,
            point: {
              ...vehicle.point,
              action: {
                ...vehicle.point?.action,
                ...(draggedMarker === "start"
                  ? { pickUpPoint: updatedLocalPos }
                  : { unLoadPoint: updatedLocalPos }),
              },
            },
          };
          return updatedVehicle;
        }
        return vehicle;
      })
    );

    setDraggedMarker(null);
    setDragOffset(null);
  };

  useEffect(() => {
    window.addEventListener("pointermove", handlePointerMove);
    window.addEventListener("pointerup", handlePointerUp);

    return () => {
      window.removeEventListener("pointermove", handlePointerMove);
      window.removeEventListener("pointerup", handlePointerUp);
    };
  }, [draggedMarker, dragOffset, isRotating]);

  if (!selectedEventSphere) {
    hasInitialized.current = false;
    return null;
  }

  return (
    <group>
      <group
        ref={startMarker}
        scale={draggedMarker === "start" ? [1.1, 1.1, 1.1] : [1, 1, 1]}
        onPointerDown={(e) => handlePointerDown(e, "start")}
      />
      <group
        ref={endMarker}
        scale={draggedMarker === "end" ? [1.1, 1.1, 1.1] : [1, 1, 1]}
        onPointerDown={(e) => handlePointerDown(e, "end")}
      />
    </group>
  );
};

export default VehicleUI;
